#!/usr/bin/env python3
from stepup.core.api import runsh, static
from stepup.reprep.api import compile_typst

static("../doc/", "../doc/main.typ", "../doc/other.typ")
compile_typst("../doc/main.typ")
runsh(
    "echo 'This sentence is generated by another step.' > ${out}",
    out="generated.typ",
    workdir="../doc",
    optional=True,
)
